@@ -56,6 +56,10 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 	polar_lines_gp = gpar(col = "#808080", lty = 3), 
 	padding = unit(5, "mm"), newpage = TRUE, vp_param = list()) {
 
+	if(inherits(xlim, c("POSIXt", "Date", "character"))) {
+		stop_wrap("`xlim` is set as a time object, please use `spiral_initialize_by_time()` instead.")
+	}
+
 	spiral_clear(check_vp = FALSE)
 
 	if(length(xlim) != 2) {
@@ -95,8 +99,7 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 
 	x = c(df1$x, rev(df2$x))
 	y = c(df1$y, rev(df2$y))
-	abs_max_x = max(abs(x))
-	abs_max_y = max(abs(y))
+	abs_max = max(abs(x), abs(y))
 
 	if(length(padding) == 1) {
 		padding = rep(padding, 2)
@@ -108,7 +111,7 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 	vp_param2 = list(name = paste0("spiral_", spiral_env$i_spiral),
 		width = unit(1, "snpc") - padding[1], 
 		height = unit(1, "snpc") - padding[2],
-		xscale = c(-abs_max_x, abs_max_x), yscale = c(-abs_max_y, abs_max_y))
+		xscale = c(-abs_max, abs_max), yscale = c(-abs_max, abs_max))
 	if("width" %in% names(vp_param)) {
 		vp_param2$width = NULL
 	}
@@ -124,7 +127,7 @@ spiral_initialize = function(xlim = c(0, 1), start = 360, end = 360*5,
 		if(d[length(d)] == 360) d = d[-length(d)]
 		dm = matrix(nrow = length(d), ncol = 4)
 		for(i in seq_along(d)) {
-			r0 = max(r + dist) + convertWidth(max(padding), "native", valueOnly = TRUE)
+			r0 = max(r + dist)# + convertWidth(max(padding), "native", valueOnly = TRUE)
 			dm[i, ] = c(0, 0, cos(d[i]/180*pi)*r0, sin(d[i]/180*pi)*r0)
 		}
 		grid.segments(dm[, 1], dm[, 2], dm[, 3], dm[, 4], default.units = "native", gp = polar_lines_gp)
@@ -351,61 +354,54 @@ spiral_initialize_by_time = function(xlim, start = NULL, end = NULL,
 
 	xlim = get_numeric_x(xlim)
 
-	if( (is.null(start) && !is.null(end)) || (!is.null(start) && is.null(end)) ) {
-		stop_wrap("You should both set 'start' and 'end'.")
-	}
-
 	start_of_the_year = as.POSIXlt("2021-01-01")
 	year(start_of_the_year) = year(time_start)
 
-	if(is.null(start)) {
-		if(unit_on_axis == "months") {
-			start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/12/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/12/period_per_loop) + start
-		} else if(unit_on_axis == "weeks") {
-			start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/52/period_per_loop)*360
-			end = 360*(time_difference(time_start, time_end, unit_on_axis)/52/period_per_loop) + start
-		} else if(unit_on_axis == "days") {
-			if(period == "years") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(52*7)/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/(52*7)/period_per_loop) + start
-			} else if(period == "months") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/30/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/30/period_per_loop) + start
-			} else if(period == "weeks") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/7/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/7/period_per_loop) + start
-			} 
-		} else if(unit_on_axis == "hours") {
-			if(period == "months") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(30*24)/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/(30*24)/period_per_loop) + start
-			} else if(period == "weeks") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(7*24)/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/(7*24)/period_per_loop) + start
-			} else if(period == "days") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/24/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/24/period_per_loop) + start
-			}
-		} else if(unit_on_axis == "mins") {
-			if(period == "days") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(24*60)/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/(24*60)/period_per_loop) + start
-			} else if(period == "hours") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
-			}
-		} else if(unit_on_axis == "secs") {
-			if(period == "hours") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(60*60)/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/(60*60)/period_per_loop) + start
-			} else if(period == "mins") {
-				start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
-				end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
-			}
+	if(unit_on_axis == "months") {
+		if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/12/period_per_loop)*360
+		end = 360*(time_difference(time_start, time_end, unit_on_axis)/12/period_per_loop) + start
+	} else if(unit_on_axis == "weeks") {
+		if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/52/period_per_loop)*360
+		end = 360*(time_difference(time_start, time_end, unit_on_axis)/52/period_per_loop) + start
+	} else if(unit_on_axis == "days") {
+		if(period == "years") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(52*7)/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(52*7)/period_per_loop) + start
+		} else if(period == "months") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/30/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/30/period_per_loop) + start
+		} else if(period == "weeks") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/7/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/7/period_per_loop) + start
+		} 
+	} else if(unit_on_axis == "hours") {
+		if(period == "months") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(30*24)/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(30*24)/period_per_loop) + start
+		} else if(period == "weeks") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(7*24)/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(7*24)/period_per_loop) + start
+		} else if(period == "days") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/24/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/24/period_per_loop) + start
+		}
+	} else if(unit_on_axis == "mins") {
+		if(period == "days") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(24*60)/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(24*60)/period_per_loop) + start
+		} else if(period == "hours") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
+		}
+	} else if(unit_on_axis == "secs") {
+		if(period == "hours") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/(60*60)/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/(60*60)/period_per_loop) + start
+		} else if(period == "mins") {
+			if(is.null(start)) start = (1 + time_difference(start_of_the_year, time_start, unit_on_axis)/60/period_per_loop)*360
+			end = 360*(time_difference(time_start, time_end, unit_on_axis)/60/period_per_loop) + start
 		}
 	}
-
 	p = 1/(xlim[2] - xlim[1])
 	start = start - (end - start)*p/2
 	end = end + (end - start)*p/2
